//
//  LSMPNSubscriptionDelegate.h
//  Lightstreamer client for iOS UCA
//

#import <Foundation/Foundation.h>


@class LSMPNSubscription;

/**
 Protocol to be implemented to receive <LSMPNSubscription> events including subscription/unsubscription, triggering and status change.
 
 Events for these delegates are dispatched by a different thread than the one that generates them. This means that, upon reception of an event,
 it is possible that the internal state of the client has changed. On the other hand, all the notifications for a single <LSLightstreamerClient>, including
 notifications to <LSClientDelegate>, <LSSubscriptionDelegate>, <LSClientMessageDelegate>, <LSMPNDeviceDelegate> and LSMPNSubscriptionDelegate
 will be dispatched by the same thread.
 */
@protocol LSMPNSubscriptionDelegate <NSObject>


@optional

/**
 @brief Event handler called when the LSMPNSubscriptionDelegate instance is added to an <LSMPNSubscription> through <[LSMPNSubscription addDelegate:]>.
 
 This is the first event to be fired on the delegate.
 
 @param subscription The <LSMPNSubscription> this instance was added to.
 */
- (void) mpnSubscriptionDidAddDelegate:(nonnull LSMPNSubscription *)subscription;

/**
 @brief Event handler called when the LSMPNSubscriptionDelegate instance is removed from an <LSMPNSubscription> through <[LSMPNSubscription removeDelegate:]>.
 
 This is the last event to be fired on the delegate.
 
 @param subscription The <LSMPNSubscription> this instance was removed from.
 */
- (void) mpnSubscriptionDidRemoveDelegate:(nonnull LSMPNSubscription *)subscription;

/**
 @brief Event handler called when an <LSMPNSubscription> has been successfully subscribed to on the server's MPN Module.
 
 This event handler is always called before other events related to the same subscription.
 
 Note that this event can be called multiple times in the life of an <LSMPNSubscription> instance only in case it is subscribed multiple times
 through <[LSLightstreamerClient unsubscribeMPN:]> and <[LSLightstreamerClient subscribeMPN:coalescing:]>. Two consecutive calls to this method
 are not possible, as before a second `mpnSubscriptionDidSubscribe:` event an <mpnSubscriptionDidUnsubscribe:> event is always fired.
 
 @param subscription The <LSMPNSubscription> involved.
 */
- (void) mpnSubscriptionDidSubscribe:(nonnull LSMPNSubscription *)subscription;

/**
 @brief Event handler called when an <LSMPNSubscription> has been successfully unsubscribed from on the server's MPN Module.
 
 After this call no more events can be received until a new <mpnSubscriptionDidSubscribe:> event.
 
 Note that this event can be called multiple times in the life of an <LSMPNSubscription> instance only in case it is subscribed multiple times
 through <[LSLightstreamerClient unsubscribeMPN:]> and <[LSLightstreamerClient subscribeMPN:coalescing:]>. Two consecutive calls to this method
 are not possible, as before a second `mpnSubscriptionDidUnsubscribe:` event an <mpnSubscriptionDidSubscribe:> event is always fired.
 
 @param subscription The <LSMPNSubscription> involved.
 */
- (void) mpnSubscriptionDidUnsubscribe:(nonnull LSMPNSubscription *)subscription;

/**
 @brief Event handler called when the server notifies an error while subscribing to an <LSMPNSubscription>.
 
 By implementing this method it is possible to perform recovery actions.
 
 The error code can be one of the following:
 
 - 17 - bad Data Adapter name or default Data Adapter not defined for the current Adapter Set.
 
 - 21 - bad Group name.
 
 - 22 - bad Group name for this Schema.
 
 - 23 - bad Schema name.
 
 - 24 - mode not allowed for an Item.
 
 - 30 - subscriptions are not allowed by the current license terms (for special licenses only).
 
 - 40 - the MPN Module is disabled, either by configuration or by license restrictions.
 
 - 41 - the request failed because of some internal resource error (e.g. database connection, timeout etc.).
 
 - 43 - invalid or unknown application ID.
 
 - 44 - invalid syntax in trigger expression.
 
 - 45 - invalid or unknown MPN device ID.
 
 - 46 - invalid or unknown MPN subscription ID (for MPN subscription modifications).
 
 - 47 - invalid argument name in notification format or trigger expression.
 
 - 48 - MPN device suspended.
 
 - 49 - one or more subscription properties exceed maximum size.
 
 - 50 - no items or fields have been specified.

 - 52 - the notification format is not a valid JSON structure.

 - 53 - the notification format is empty.

 - 66 - an unexpected exception was thrown by the Metadata Adapter while authorizing the connection.
 
 - 68 - the Server could not fulfill the request because of an internal error.
 
 - &lt;= 0 - the Metadata Adapter has refused the subscription request; the code value is dependent on the specific Metadata Adapter implementation.
 
 @param subscription The <LSMPNSubscription> involved.
 
 @param code The error code sent by the Server.
 
 @param message The description of the error sent by the Server; it can be nil.
 */
- (void) mpnSubscription:(nonnull LSMPNSubscription *)subscription didFailSubscriptionWithErrorCode:(NSInteger)code message:(nullable NSString *)message;

/**
 @brief Event handler called when the server notifies an error while unsubscribing from an <LSMPNSubscription>.
 
 By implementing this method it is possible to perform recovery actions.
 
 The error code can be one of the following:
 
 - 30 - subscriptions are not allowed by the current license terms (for special licenses only).
 
 - 40 - the MPN Module is disabled, either by configuration or by license restrictions.
 
 - 41 - the request failed because of some internal resource error (e.g. database connection, timeout etc.).
 
 - 43 - invalid or unknown application ID.
 
 - 45 - invalid or unknown MPN device ID.
 
 - 46 - invalid or unknown MPN subscription ID.
 
 - 48 - MPN device suspended.
 
 - 66 - an unexpected exception was thrown by the Metadata Adapter while authorizing the connection.
 
 - 68 - the Server could not fulfill the request because of an internal error.
 
 - &lt;= 0 - the Metadata Adapter has refused the unsubscription request; the code value is dependent on the specific Metadata Adapter implementation.
 
 @param subscription The <LSMPNSubscription> involved.
 
 @param code The error code sent by the Server.
 
 @param message The description of the error sent by the Server; it can be nil.
 */
- (void) mpnSubscription:(nonnull LSMPNSubscription *)subscription didFailUnsubscriptionWithErrorCode:(NSInteger)code message:(nullable NSString *)message;

/**
 @brief Event handler called when the server notifies that an <LSMPNSubscription> did trigger.
 
 For this event to be called the <LSMPNSubscription> must have a <[LSMPNSubscription triggerExpression]> set and it must have been evaluated to true at
 least once.
 
 Note that this event can be called multiple times in the life of an <LSMPNSubscription> instance only in case it is subscribed multiple times
 through <[LSLightstreamerClient unsubscribeMPN:]> and <[LSLightstreamerClient subscribeMPN:coalescing:]>. Two consecutive calls to this method
 are not possible.
 
 Note also that in some server clustering configurations this event may not be called. The corrisponding push notification is always sent, though.

 @param subscription The <LSMPNSubscription> involved.

 @see [LSMPNSubscription triggerExpression]
 */
- (void) mpnSubscriptionDidTrigger:(nonnull LSMPNSubscription *)subscription;

/**
 @brief Event handler called when the server notifies that an <LSMPNSubscription> changed its status.
 
 Note that in some server clustering configurations the status change for the MPN subscription's trigger event may not be called. The corrisponding push
 notification is always sent, though.
 
 @param subscription The <LSMPNSubscription> involved.
 @param status The new status of the MPN subscription.
 @param timestamp The server-side timestamp of the new subscription status.

 @see [LSMPNSubscription status]
 @see [LSMPNSubscription statusTimestamp]
 */
- (void) mpnSubscription:(nonnull LSMPNSubscription *)subscription didChangeStatus:(nonnull NSString *)status timestamp:(nonnull NSDate *)timestamp;

/**
 Event handler called each time the value of a property of <LSMPNSubscription> is changed.
 
 Properties can be modified by direct calls to them or by server sent events. A propery may be changed by a server sent event when the MPN subscription is
 modified, or when two MPN subscriptions coalesce (see <[LSLightstreamerClient subscribeMPN:coalescing:]>).
 
 Possible property names are the following:
 
 - `mode`
 
 - `group`
 
 - `schema`
 
 - `adapter`
 
 - `notification_format`
 
 - `trigger`
 
 - `requested_buffer_size`
 
 - `requested_max_frequency`
 
 - `status_timestamp`
 
 @param subscription The <LSMPNSubscription> involved.

 @param property The name of the changed property.
 */
- (void) mpnSubscription:(nonnull LSMPNSubscription *)subscription didChangeProperty:(nonnull NSString *)property;


@end
